@@ -19,6 +19,7 @@
 #include "main.h"
 #include "prefdb.h"
 #include <glibmm/i18n.h>
+#include <X11/XKBlib.h>
 #include "grabber.h"
 
 #include <typeinfo>
@@ -89,14 +90,7 @@ protected:
 				return true;
 		}
 		guint mods = event->state & gtk_accelerator_get_default_mod_mask();
-		guint key;
-		if (mods & ~GDK_SHIFT_MASK) {
-			key = XKeycodeToKeysym(dpy, event->hardware_keycode, 0);
-		} else {
-			key = gdk_keyval_to_lower(event->keyval);
-			if (key == event->keyval)
-				mods = 0;
-		}
+		guint key = XkbKeycodeToKeysym(dpy, event->hardware_keycode, 0, 0);
 
 		editing_done();
 		remove_widget();
@@ -191,6 +185,7 @@ const char *type_info_to_name(const std::type_info *info) {
 
 Actions::Actions() :
 	apps_view(0),
+	vpaned_position(-1),
 	editing_new(false),
 	editing(false),
 	action_list(actions.get_root())
@@ -211,6 +206,7 @@ Actions::Actions() :
 	widgets->get_widget("button_reset_actions", button_reset_actions);
 	widgets->get_widget("check_show_deleted", check_show_deleted);
 	widgets->get_widget("expander_apps", expander_apps);
+	widgets->get_widget("vpaned_apps", vpaned_apps);
 	button_record->signal_clicked().connect(sigc::mem_fun(*this, &Actions::on_button_record));
 	button_delete->signal_clicked().connect(sigc::mem_fun(*this, &Actions::on_button_delete));
 	button_add->signal_clicked().connect(sigc::mem_fun(*this, &Actions::on_button_new));
@@ -219,7 +215,6 @@ Actions::Actions() :
 	button_remove_app->signal_clicked().connect(sigc::mem_fun(*this, &Actions::on_remove_app));
 	button_reset_actions->signal_clicked().connect(sigc::mem_fun(*this, &Actions::on_reset_actions));
 
-	tv.signal_cursor_changed().connect(sigc::mem_fun(*this, &Actions::on_cursor_changed));
 	tv.signal_row_activated().connect(sigc::mem_fun(*this, &Actions::on_row_activated));
 	tv.get_selection()->signal_changed().connect(sigc::mem_fun(*this, &Actions::on_selection_changed));
 
@@ -283,6 +278,7 @@ Actions::Actions() :
 
 	check_show_deleted->signal_toggled().connect(sigc::mem_fun(*this, &Actions::update_action_list));
 	expander_apps->property_expanded().signal_changed().connect(sigc::mem_fun(*this, &Actions::on_apps_selection_changed));
+	expander_apps->property_expanded().signal_changed().connect(sigc::mem_fun(*this, &Actions::on_expanded));
 	apps_view->get_selection()->signal_changed().connect(sigc::mem_fun(*this, &Actions::on_apps_selection_changed));
 	apps_model = AppsStore::create(ca, this);
 
@@ -685,6 +681,18 @@ void Actions::on_group_name_edited(const Glib::ustring& path, const Glib::ustrin
 	update_actions();
 }
 
+void Actions::on_expanded() {
+	if (expander_apps->get_expanded()) {
+		vpaned_apps->set_position(vpaned_position);
+	} else {
+		if(vpaned_apps->property_position_set().get_value())
+			vpaned_position = vpaned_apps->get_position();
+		else
+			vpaned_position = -1;
+		vpaned_apps->property_position_set().set_value(false);
+	}
+}
+
 void Actions::on_apps_selection_changed() {
 	ActionListDiff *new_action_list = actions.get_root();
 	if (expander_apps->property_expanded().get_value()) {
@@ -711,6 +719,7 @@ void Actions::update_counts() {
 }
 
 void Actions::update_action_list() {
+	check_show_deleted->set_sensitive(action_list != actions.get_root());
 	boost::shared_ptr<std::set<Unique *> > ids = action_list->get_ids(check_show_deleted->get_active());
 	const Gtk::TreeNodeChildren &ch = tm->children();
 
@@ -751,7 +760,7 @@ void Actions::update_row(const Gtk::TreeRow &row) {
 }
 
 extern boost::shared_ptr<sigc::slot<void, RStroke> > stroke_action;
-Source<ActionListDiff *> stroke_app(NULL);
+Source<bool> recording(false);
 
 class Actions::OnStroke {
 	Actions *parent;
@@ -787,7 +796,7 @@ public:
 		stroke = stroke_;
 		Glib::signal_idle().connect(sigc::mem_fun(*this, &OnStroke::run));
 		stroke_action.reset();
-		stroke_app.set(NULL);
+		recording.set(false);
 	}
 };
 
@@ -815,14 +824,14 @@ void Actions::on_row_activated(const Gtk::TreeModel::Path& path, Gtk::TreeViewCo
 
 	OnStroke ps(this, dialog, row);
 	stroke_action.reset(new sigc::slot<void, RStroke>(sigc::mem_fun(ps, &OnStroke::delayed_run)));
-	stroke_app.set(action_list);
+	recording.set(true);
 
 	dialog->show();
 	cancel->grab_focus();
 	int response = dialog->run();
 	dialog->hide();
 	stroke_action.reset();
-	stroke_app.set(NULL);
+	recording.set(false);
 	if (response != 1)
 		return;
 
@@ -839,13 +848,6 @@ void Actions::on_button_record() {
 	on_row_activated(path, col);
 }
 
-void Actions::on_cursor_changed() {
-	Gtk::TreeModel::Path path;
-	Gtk::TreeViewColumn *col;
-	tv.get_cursor(path, col);
-	Gtk::TreeRow row(*tm->get_iter(path));
-}
-
 Gtk::TreeRow Actions::get_selected_row() {
 	std::vector<Gtk::TreePath> paths = tv.get_selection()->get_selected_rows();
 	return Gtk::TreeRow(*tm->get_iter(*paths.begin()));
