@@ -16,8 +16,7 @@
 #include "actiondb.h" // TODO
 #include "grabber.h"
 #include "main.h"
-#include "win.h" // TODO
-#include "prefs.h" // TODO
+#include "prefs.h"
 #include <X11/extensions/XTest.h>
 #include <xorg/xserver-properties.h>
 #include <X11/cursorfont.h>
@@ -27,8 +26,7 @@
 extern Window get_window(Window w, Atom prop);
 extern Source<bool> disabled;
 extern Source<Window> current_app_window;
-extern Source<ActionListDiff *> stroke_app;
-extern bool in_proximity;
+extern Source<bool> recording;
 
 Grabber *grabber = 0;
 
@@ -189,9 +187,7 @@ void activate(Window w, Time t) {
 	XSendEvent(dpy, ROOT, False, SubstructureNotifyMask | SubstructureRedirectMask, (XEvent *)&ev);
 }
 
-std::string get_wm_class(Window w, ActionListDiff *actions) {
-	if (actions && actions->app)
-		return actions->name;
+std::string get_wm_class(Window w) {
 	if (!w)
 		return "";
 	XClassHint ch;
@@ -236,8 +232,9 @@ Grabber::Grabber() : children(ROOT) {
 	init_xi();
 	prefs.excluded_devices.connect(new IdleNotifier(sigc::mem_fun(*this, &Grabber::update)));
 	prefs.button.connect(new IdleNotifier(sigc::mem_fun(*this, &Grabber::update)));
-	current_class = fun2(&get_wm_class, current_app_window, stroke_app);
+	current_class = fun(&get_wm_class, current_app_window);
 	current_class->connect(new IdleNotifier(sigc::mem_fun(*this, &Grabber::update)));
+	recording.connect(new IdleNotifier(sigc::mem_fun(*this, &Grabber::update)));
 	disabled.connect(new IdleNotifier(sigc::mem_fun(*this, &Grabber::set)));
 	update();
 	resume();
@@ -309,32 +306,33 @@ bool Grabber::init_xi() {
 	return true;
 }
 
-void Grabber::hierarchy_changed(XIHierarchyEvent *event) {
+bool Grabber::hierarchy_changed(XIHierarchyEvent *event) {
+	bool changed = false;
 	for (int i = 0; i < event->num_info; i++) {
 		XIHierarchyInfo *info = event->info + i;
 		if (info->flags & XISlaveAdded) {
 			int n;
 			XIDeviceInfo *dev_info = XIQueryDevice(dpy, info->deviceid, &n);
 			if (!dev_info)
-				return;
+				continue;
 			new_device(dev_info);
 			XIFreeDeviceInfo(dev_info);
 			update_excluded();
-			win->prefs_tab->update_device_list();
+			changed = true;
 		} else if (info->flags & XISlaveRemoved) {
 			if (verbosity >= 1)
 				printf("Device %d removed.\n", info->deviceid);
 			xi_devs.erase(info->deviceid);
-			win->prefs_tab->update_device_list();
 			if (current_dev && current_dev->dev == info->deviceid)
 				current_dev = NULL;
+			changed = true;
 		} else if (info->flags & (XISlaveAttached | XISlaveDetached)) {
 			DeviceMap::iterator i = xi_devs.find(info->deviceid);
-			if (i == xi_devs.end())
-				return;
-			i->second->master = info->attachment;
+			if (i != xi_devs.end())
+				i->second->master = info->attachment;
 		}
 	}
+	return changed;
 }
 
 void Grabber::update_excluded() {
@@ -373,7 +371,7 @@ void Grabber::new_device(XIDeviceInfo *info) {
 		}
 }
 
-Grabber::XiDevice::XiDevice(Grabber *parent, XIDeviceInfo *info) : supports_pressure(false), absolute(false), scale_x(1.0), scale_y(1.0), num_buttons(0) {
+Grabber::XiDevice::XiDevice(Grabber *parent, XIDeviceInfo *info) : absolute(false), scale_x(1.0), scale_y(1.0), num_buttons(0) {
 	dev = info->deviceid;
 	name = info->name;
 	master = info->attachment;
@@ -392,20 +390,11 @@ Grabber::XiDevice::XiDevice(Grabber *parent, XIDeviceInfo *info) : supports_pres
 					scale_y = (double)DisplayHeight(dpy, DefaultScreen(dpy)) / (double)(v->max - v->min);
 
 			}
-			if (v->number == 2) {
-				pressure_min = v->min;
-				pressure_max = v->max;
-				if (pressure_min < pressure_max)
-					supports_pressure = true;
-			}
 		}
 	}
 
 	if (verbosity >= 1)
-		printf("Opened Device %d ('%s'%s).\n", dev, info->name,
-				absolute ?
-					supports_pressure ? ": absolute, pressure" : ": absolute" :
-					supports_pressure ? ": pressure" : "");
+		printf("Opened Device %d ('%s'%s).\n", dev, info->name, absolute ? ": absolute" : "");
 }
 
 Grabber::XiDevice *Grabber::get_xi_dev(int id) {
@@ -512,28 +501,28 @@ bool Grabber::is_click_hold(guint b) {
 	return false;
 }
 
-int get_default_button() {
-	if (grabber)
-		return grabber->get_default_button();
-	else
-		return prefs.button.get().button;
+guint Grabber::get_default_mods(guint button) {
+	for (std::vector<ButtonInfo>::const_iterator i = buttons.begin(); i != buttons.end(); ++i)
+		if (i->button == button)
+			return i->state;
+	return AnyModifier;
 }
 
 void Grabber::update() {
-	std::map<std::string, RButtonInfo>::const_iterator i = prefs.exceptions.ref().find(current_class->get());
-	active = true;
 	ButtonInfo bi = prefs.button.ref();
-	if (i != prefs.exceptions.ref().end()) {
-		if (i->second)
-			bi = *i->second;
-		else
+	active = true;
+	if (!recording.get()) {
+		std::map<std::string, RButtonInfo>::const_iterator i = prefs.exceptions.ref().find(current_class->get());
+		if (i != prefs.exceptions.ref().end()) {
+			if (i->second)
+				bi = *i->second;
+			else
+				active = false;
+		}
+
+		if (prefs.whitelist.get() && !actions.apps.count(current_class->get()))
 			active = false;
 	}
-	const ActionListDiff *a = stroke_app.get();
-	if (!a)
-		a = actions.get_action_list(current_class->get());
-	if (active && actions.get_root()->size_rec() && !a->count_actions())
-		active = false;
 	const std::vector<ButtonInfo> &extra = prefs.extra_buttons.ref();
 	if (grabbed_button == bi && buttons.size() == extra.size() + 1 &&
 			std::equal(extra.begin(), extra.end(), ++buttons.begin())) {
