@@ -32,6 +32,7 @@
 #include <X11/extensions/XTest.h>
 #include <X11/extensions/Xfixes.h>
 #include <X11/Xproto.h>
+#include <X11/XKBlib.h>
 // From #include <X11/extensions/XIproto.h>
 // which is not C++-safe
 #define X_GrabDeviceButton              17
@@ -46,7 +47,7 @@ extern Source<bool> disabled;
 bool experimental = false;
 int verbosity = 0;
 const char *prefs_versions[] = { "-0.5.5", "-0.4.1", "-0.4.0", "", NULL };
-const char *actions_versions[] = { "-0.4.1", "-0.4.0", "", NULL };
+const char *actions_versions[] = { "-0.5.6", "-0.4.1", "-0.4.0", "", NULL };
 Source<Window> current_app_window(None);
 std::string config_dir;
 Win *win = NULL;
@@ -56,6 +57,7 @@ bool in_proximity = false;
 boost::shared_ptr<sigc::slot<void, RStroke> > stroke_action;
 Grabber::XiDevice *current_dev = 0;
 std::set<guint> xinput_pressed; // TODO get rid of
+guint modifiers;
 
 static bool show_gui = false;
 static bool no_dbus = false;
@@ -139,7 +141,6 @@ public:
 	virtual void press(guint b, RTriple e) {}
 	virtual void release(guint b, RTriple e) {}
 	virtual void press_master(guint b, Time t) {}
-	virtual void pressure() {}
 	virtual void proximity_out() {}
 	virtual void pong() {}
 	void replace_child(Handler *c) {
@@ -818,6 +819,7 @@ static void get_timeouts(TimeoutType type, int *init, int *final) {
 
 class StrokeHandler : public Handler, public sigc::trackable {
 	guint button;
+	guint trigger;
 	RPreStroke cur;
 	bool is_gesture;
 	bool drawing;
@@ -842,9 +844,7 @@ class StrokeHandler : public Handler, public sigc::trackable {
 		RPreStroke c = cur;
 		if (!is_gesture || grabber->is_instant(button))
 			c.reset(new PreStroke);
-		if (b && prefs.advanced_ignore.get())
-			c.reset(new PreStroke);
-		return Stroke::create(*c, button, b, false);
+		return Stroke::create(*c, trigger, b, modifiers, false);
 	}
 
 	bool timeout() {
@@ -856,7 +856,7 @@ class StrokeHandler : public Handler, public sigc::trackable {
 			c.reset(new PreStroke);
 		RStroke s;
 		if (prefs.timeout_gestures.get() || grabber->is_click_hold(button))
-			s = Stroke::create(*c, button, 0, true);
+			s = Stroke::create(*c, trigger, 0, modifiers, true);
 		parent->replace_child(AdvancedHandler::create(s, last, button, 0, cur));
 		XFlush(dpy);
 		return false;
@@ -864,7 +864,7 @@ class StrokeHandler : public Handler, public sigc::trackable {
 
 	void do_instant() {
 		PreStroke ps;
-		RStroke s = Stroke::create(ps, button, button, false);
+		RStroke s = Stroke::create(ps, trigger, button, modifiers, false);
 		parent->replace_child(AdvancedHandler::create(s, orig, button, button, cur));
 	}
 
@@ -876,10 +876,6 @@ protected:
 	void abort_stroke() {
 		parent->replace_child(AdvancedHandler::create(RStroke(), last, button, 0, cur));
 	}
-	virtual void pressure() {
-		abort_stroke();
-		timeout();
-	}
 	virtual void motion(RTriple e) {
 		cur->add(e);
 		float dist = hypot(e->x-orig->x, e->y-orig->y);
@@ -940,7 +936,7 @@ protected:
 		if (!IS_CLICK(act))
 			Ranking::queue_show(ranking, e);
 		if (!act) {
-			XBell(dpy, 0);
+			XkbBell(dpy, None, 0, None);
 			return parent->replace_child(NULL);
 		}
 		RModifiers mods = act->prepare();
@@ -969,8 +965,17 @@ protected:
 		parent->replace_child(NULL);
 	}
 public:
-	StrokeHandler(guint b, RTriple e) : button(b), is_gesture(false), drawing(false), last(e), orig(e),
-	init_timeout(prefs.init_timeout.get()), final_timeout(prefs.final_timeout.get()), radius(16) {
+	StrokeHandler(guint b, RTriple e) :
+		button(b),
+		trigger(grabber->get_default_button() == (int)b ? 0 : b),
+		is_gesture(false),
+		drawing(false),
+		last(e),
+		orig(e),
+		init_timeout(prefs.init_timeout.get()),
+		final_timeout(prefs.final_timeout.get()),
+		radius(16)
+	{
 		const std::map<std::string, TimeoutType> &dt = prefs.device_timeout.ref();
 		std::map<std::string, TimeoutType>::const_iterator j = dt.find(current_dev->name);
 		if (j != dt.end())
@@ -1082,6 +1087,7 @@ void quit() {
 	if (dead)
 		bail_out();
 	dead = true;
+	win->hide();
 	queue(sigc::ptr_fun(&Gtk::Main::quit));
 }
 
@@ -1130,7 +1136,7 @@ static void link_button_hook(Gtk::LinkButton *, const Glib::ustring& uri) { xdg_
 static void about_dialog_hook(Gtk::AboutDialog &, const Glib::ustring& url) { xdg_open(url); }
 
 // dbus-send --type=method_call --dest=org.easystroke /org/easystroke org.easystroke.send string:"foo"
-static void send_dbus(const char *str) {
+static void send_dbus(bool action, const char *str) {
 	GError *error = 0;
 	DBusGConnection *bus = dbus_g_bus_get(DBUS_BUS_SESSION, &error);
 	if (!bus) {
@@ -1138,7 +1144,10 @@ static void send_dbus(const char *str) {
 		exit(EXIT_FAILURE);
 	}
 	DBusGProxy *proxy = dbus_g_proxy_new_for_name(bus, "org.easystroke", "/org/easystroke", "org.easystroke");
-	dbus_g_proxy_call_no_reply(proxy, "send", G_TYPE_STRING, str, G_TYPE_INVALID);
+	if (action)
+		dbus_g_proxy_call_no_reply(proxy, "send", G_TYPE_STRING, str, G_TYPE_INVALID);
+	else
+		dbus_g_proxy_call_no_reply(proxy, str, G_TYPE_INVALID);
 }
 
 int start_dbus();
@@ -1156,7 +1165,13 @@ Main::Main() : kit(0) {
 		if (argc == 2)
 			usage(argv[0], false);
 		gtk_init(&argc, &argv);
-		send_dbus(argv[2]);
+		send_dbus(true, argv[2]);
+		exit(EXIT_SUCCESS);
+	}
+
+	if (argc > 1 && (!strcmp(argv[1], "enable") || !strcmp(argv[1], "about") || !strcmp(argv[1], "quit"))) {
+		gtk_init(&argc, &argv);
+		send_dbus(false, argv[1]);
 		exit(EXIT_SUCCESS);
 	}
 
@@ -1177,7 +1192,7 @@ Main::Main() : kit(0) {
 	}
 	if (!no_dbus && start_dbus() < 0) {
 		printf(_("Easystroke is already running, showing configuration window instead.\n"));
-		send_dbus("");
+		send_dbus(true, "");
 		exit(EXIT_SUCCESS);
 	}
 	ROOT = DefaultRootWindow(dpy);
@@ -1233,6 +1248,9 @@ void Main::usage(char *me, bool good) {
 	printf("\n");
 	printf("Usage: %s [OPTION]...\n", me);
 	printf("or:    %s send <action_name>\n", me);
+	printf("       %s enable\n", me);
+	printf("       %s about\n", me);
+	printf("       %s quit\n", me);
 	printf("\n");
 	printf("Options:\n");
 	printf("  -c, --config-dir       Directory for config files\n");
@@ -1398,9 +1416,10 @@ void Main::handle_event(XEvent &ev) {
 		return;
 
 	case MappingNotify:
-		if (ev.xmapping.request != MappingPointer)
-			return;
-		update_core_mapping();
+		if (ev.xmapping.request == MappingPointer)
+			update_core_mapping();
+		if (ev.xmapping.request == MappingKeyboard || ev.xmapping.request == MappingModifier)
+			XRefreshKeyboardMapping(&ev.xmapping);
 		return;
 	case GenericEvent:
 		if (ev.xcookie.extension == grabber->opcode && XGetEventData(dpy, &ev.xcookie)) {
@@ -1448,6 +1467,10 @@ void Main::handle_xi2_event(XIDeviceEvent *event) {
 			if (xinput_pressed.size()) {
 				if (!current_dev || current_dev->dev != event->deviceid)
 					break;
+			} else {
+				current_app_window.set(get_app_window(event->child));
+				if (verbosity >= 3)
+					printf("Active window 0x%lx -> 0x%lx\n", event->child, current_app_window.get());
 			}
 			current_dev = grabber->get_xi_dev(event->deviceid);
 			if (!current_dev) {
@@ -1456,6 +1479,13 @@ void Main::handle_xi2_event(XIDeviceEvent *event) {
 			}
 			if (current_dev->master)
 				XISetClientPointer(dpy, None, current_dev->master);
+			if (!xinput_pressed.size()) {
+				guint default_mods = grabber->get_default_mods(event->detail);
+				if (default_mods == AnyModifier || default_mods == (guint)event->mods.base)
+					modifiers = AnyModifier;
+				else
+					modifiers = event->mods.base;
+			}
 			xinput_pressed.insert(event->detail);
 			H->press(event->detail, create_triple(event->root_x, event->root_y, event->time));
 			break;
@@ -1472,23 +1502,14 @@ void Main::handle_xi2_event(XIDeviceEvent *event) {
 				report_xi2_event(event, "Motion");
 			if (!current_dev || current_dev->dev != event->deviceid)
 				break;
-			if (current_dev->supports_pressure && XIMaskIsSet(event->valuators.mask, 2)) {
-				int i = 0;
-				if (XIMaskIsSet(event->valuators.mask, 0))
-				       i++;
-				if (XIMaskIsSet(event->valuators.mask, 1))
-				       i++;
-				int z = current_dev->normalize_pressure(event->valuators.values[i]);
-				if (prefs.pressure_abort.get() && z >= prefs.pressure_threshold.get())
-					H->pressure();
-			}
 			H->motion(create_triple(event->root_x, event->root_y, event->time));
 			break;
 		case XI_RawMotion:
 			handle_raw_motion((XIRawEvent *)event);
 			break;
 		case XI_HierarchyChanged:
-			grabber->hierarchy_changed((XIHierarchyEvent *)event);
+			if (grabber->hierarchy_changed((XIHierarchyEvent *)event))
+				win->prefs_tab->update_device_list();
 	}
 }
 
@@ -1616,17 +1637,20 @@ bool fake_char(gunichar c) {
 	if (!keycode)
 		return false;
 	KeyCode modifier = 0;
-	if (XKeycodeToKeysym(dpy, keycode, 0) != keysym) {
+	int n;
+	KeySym *mapping = XGetKeyboardMapping(dpy, keycode, 1, &n);
+	if (mapping[0] != keysym) {
 		int i;
-		for (i = 1; i < 8; i++)
-			if (XKeycodeToKeysym(dpy, keycode, i) == keysym)
+		for (i = 1; i < n; i++)
+			if (mapping[i] == keysym)
 				break;
-		if (i == 8)
+		if (i == n)
 			return false;
 		XModifierKeymap *keymap = XGetModifierMapping(dpy);
 		modifier = keymap->modifiermap[i];
 		XFreeModifiermap(keymap);
 	}
+	XFree(mapping);
 	if (modifier)
 		XTestFakeKeyEvent(dpy, modifier, true, 0);
 	XTestFakeKeyEvent(dpy, keycode, true, 0);
