@@ -69,8 +69,15 @@ void Stroke::draw(Cairo::RefPtr<Cairo::Surface> surface, int x, int y, int w, in
 	}
 	ctx->restore();
 	Glib::ustring str;
+	if (modifiers != AnyModifier) {
+		str = Gtk::AccelGroup::get_label(0, (Gdk::ModifierType)modifiers);
+		if (str == "")
+			str = "<>";
+		else
+			str = "<" + str.substr(0, str.size()-1) + ">";
+	}
 	if (trigger)
-		str = Glib::ustring::compose("%1\xE2\x86\x92", trigger);
+		str += Glib::ustring::compose("%1\xE2\x86\x92", trigger);
 	if (timeout)
 		str += "x";
 	if (button)
@@ -81,9 +88,15 @@ void Stroke::draw(Cairo::RefPtr<Cairo::Surface> surface, int x, int y, int w, in
 		ctx->set_source_rgba(0.0, 1.0, 1.0, 0.8);
 	else
 		ctx->set_source_rgba(1.0, 0.0, 0.0, 0.8);
-	ctx->set_font_size(h*0.5);
+	float font_size = h*0.5;
 	Cairo::TextExtents te;
-	ctx->get_text_extents(str, te);
+	for (;;) {
+		ctx->set_font_size(font_size);
+		ctx->get_text_extents(str, te);
+		if (te.width < w)
+			break;
+		font_size *= 0.9;
+	}
 	ctx->move_to(x+w/2 - te.x_bearing - te.width/2, y+h/2 - te.y_bearing - te.height/2);
 	ctx->show_text(str);
 }
@@ -164,7 +177,7 @@ Win::Win() : actions(new Actions), prefs_tab(new Prefs), stats(new Stats) {
 
 	WIDGET(Gtk::ImageMenuItem, menu_about, Gtk::Stock::ABOUT);
 	menu.append(menu_about);
-	menu_about.signal_activate().connect(sigc::mem_fun(*this, &Win::on_about));
+	menu_about.signal_activate().connect(sigc::mem_fun(*this, &Win::show_about));
 
 	WIDGET(Gtk::SeparatorMenuItem, menu_sep);
 	menu.append(menu_sep);
@@ -223,7 +236,7 @@ void Win::show_popup(guint button, guint32 activate_time) {
 }
 
 extern const char *version_string;
-void Win::on_about() {
+void Win::show_about() {
 	Gtk::AboutDialog *about;
 	widgets->get_widget("aboutdialog", about);
 	about->set_logo(Stroke::trefoil()->draw(96, 4.0));
@@ -241,6 +254,10 @@ void Win::show_hide() {
 		win->show();
 }
 
+void Win::hide() {
+	win->hide();
+}
+
 bool Win::on_icon_size_changed(int size) {
 	icon_pb[0] = Stroke::trefoil()->draw(size);
 	icon_pb[1] = Stroke::trefoil()->draw(size);
@@ -269,5 +286,5 @@ void error_dialog(const Glib::ustring &text) {
 }
 
 Glib::ustring app_name_hr(Glib::ustring src) {
-	return src == "" ? _("(window manager frame)") : src;
+	return src == "" ? _("<unnamed>") : src;
 }
